import boto3
import boto3.session
import glob
import os
import urllib3

urllib3.disable_warnings()

def conexao_s3():
    session = boto3.session.Session(aws_access_key_id=os.getenv("DETIC_ACCESS_KEY_ID"),
                                    aws_secret_access_key=os.getenv("DETIC_SECRET_ACCESS_KEY"))
    endpoint = 'https://s3.nuvem.unicamp.br'
    s3 = session.resource(service_name='s3', endpoint_url=endpoint, verify=False)
    return s3

def listar_objetos_s3(sessao,bucket_name):
    client = sessao.meta.client
    bucket = sessao.Bucket(bucket_name)
    registros = []
    lista = bucket.objects.all()
    for o in lista:
        #capturando metadados do objeto
        response = sessao.Object(bucket_name, o.key).get()
        linha = {}
        linha['bucket'] = bucket
        linha['nome_objeto'] = o.key
        linha['tamanho_objeto'] = o.size
        lista = tuple(linha.values())
        registros.append(lista)

    return registros

def upload_objetos_s3(sessao, bucket, origem, destino):
    rel_paths = glob.glob(origem + '/**', recursive=True)
    for local_file in rel_paths:
        if not os.path.isdir(local_file):
            local_file_linux = local_file.replace(os.sep, "/")
            if destino == '/':
                destino = ''
            sessao.Object(bucket, destino + local_file_linux[0 + len(origem):]).upload_file(
                local_file)
            
def deletar_objetos_s3(sessao,bucket, diretorio):
    for o in sessao.Bucket(bucket).objects.filter(Prefix='/' + diretorio):
        print(o.key)
        sessao.Object(bucket, o.key).delete()

def download_objeto_s3(sessao, bucket, arquivo, diretorio):
    path = diretorio + arquivo
    sessao.Object(bucket, arquivo).download_file(path)


import pandas as pd
from sqlalchemy.dialects.postgresql import insert
from sqlalchemy import Table
from sqlalchemy.engine.base import Engine as sql_engine
from sqlalchemy.ext.automap import automap_base


def upsert_database(df_input: pd.DataFrame, engine: sql_engine, table: str, schema: str) -> None:
    """
    Realiza um upsert dos dados contidos no dataframe 'df_input' para a tabela 'table'
    :param self:
    :param df_input: dataframe com dados a serem salvos/atualizados
    :param engine: engine do bd
    :param table: tabela do bd a ser atualizada
    :param schema: esquema do bd
    :return:
    """

    if df_input is None or len(df_input) == 0:
        return None
    flattened_input = df_input.to_dict('records')
    with engine.connect() as conn:
        base = automap_base()
        base.prepare(engine, reflect=True, schema=schema)
        target_table = Table(table, base.metadata,
                             autoload=True, autoload_with=engine, schema=schema)
        chunks = [flattened_input[i:i + 1000] for i in range(0, len(flattened_input), 1000)]
        for chunk in chunks:
            stmt = insert(target_table).values(chunk)
            update_dict = {c.name: c for c in stmt.excluded if not c.primary_key}
            conn.execute(stmt.on_conflict_do_update(
                constraint=f'{table}_pk',
                set_=update_dict)
            )